unit MainCont;

interface

uses
  YahooCont, Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Spin, IniFiles;

type
  TMainFormCont = class(TForm)
    ConnectBtn: TButton;
    GroupBox1: TGroupBox;
    ContactsLB: TListBox;
    ConAddBtn: TButton;
    ConEditBtn: TButton;
    ConDeleteBtn: TButton;
    GroupBox2: TGroupBox;
    GroupsLB: TListBox;
    GrAddBtn: TButton;
    GrEditBtn: TButton;
    GrDeleteBtn: TButton;
    Label3: TLabel;
    Label4: TLabel;
    ConAddGrBtn: TButton;
    ConRemGrBtn: TButton;
    CheckBox1: TCheckBox;
    ProxyEd: TEdit;
    Label5: TLabel;
    PortEd: TSpinEdit;
    StoreBtn: TButton;
    procedure ConnectBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ContactsLBClick(Sender: TObject);
    procedure ConEditBtnClick(Sender: TObject);
    procedure ConAddBtnClick(Sender: TObject);
    procedure ConDeleteBtnClick(Sender: TObject);
    procedure GroupsLBClick(Sender: TObject);
    procedure GrAddBtnClick(Sender: TObject);
    procedure GrEditBtnClick(Sender: TObject);
    procedure GrDeleteBtnClick(Sender: TObject);
    procedure GroupsLBDblClick(Sender: TObject);
    procedure ContactsLBDblClick(Sender: TObject);
    procedure ConAddGrBtnClick(Sender: TObject);
    procedure ConRemGrBtnClick(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure StoreBtnClick(Sender: TObject);
  private
    IniFile: TMemIniFile;
    procedure UpdateContacts(Group: TYGroup);
    procedure UpdateConButtons;
    procedure UpdateGroups;
    procedure UpdateGrButtons;
  end;

var
  MainFormCont: TMainFormCont;

implementation

uses EditCont, EditGroup, SelGroup;

{$R *.dfm}

var
  YContacts: TYContacts;

procedure TMainFormCont.ConnectBtnClick(Sender: TObject);
begin
  ConAddBtn.Enabled := False;
  GrAddBtn.Enabled := False;
  GroupsLB.Items.Clear;
  ContactsLB.Items.Clear;
  Application.ProcessMessages;
  if not CheckBox1.Checked then
  begin
    YContacts.ProxyServer := ProxyEd.Text;
    YContacts.ProxyPort := PortEd.Value;
  end
  else
  begin
    YContacts.ProxyServer := '';
    YContacts.ProxyPort := 0;
  end;
  if not YContacts.Load then
    Exit;
  ConAddBtn.Enabled := True;
  GrAddBtn.Enabled := True;
  UpdateGroups;
  UpdateContacts(nil);
  ConnectBtn.Caption := 'Refresh';
end;

procedure TMainFormCont.FormCreate(Sender: TObject);
begin
  YContacts := TYContacts.Create;
  YContacts.ConsumerKey := ''; // Enter your consumer key here
  YContacts.ConsumerSecret := ''; // Enter your consumer secret key here
  IniFile := TMemIniFile.Create('LastSession.ini');
  YContacts.Token := IniFile.ReadString('Session', 'Token', '');
  YContacts.TokenSecret := IniFile.ReadString('Session', 'TokenSecret', '');
  YContacts.SessionHandle := IniFile.ReadString('Session', 'SessionHandle', '');
end;

procedure TMainFormCont.FormDestroy(Sender: TObject);
begin
  IniFile.WriteString('Session', 'Token', YContacts.Token);
  IniFile.WriteString('Session', 'TokenSecret', YContacts.TokenSecret);
  IniFile.WriteString('Session', 'SessionHandle', YContacts.SessionHandle);
  IniFile.UpdateFile;
  IniFile.Free;
  YContacts.Free;
end;

procedure TMainFormCont.UpdateConButtons;
begin
  ConEditBtn.Enabled := ContactsLB.ItemIndex >= 0;
  ConDeleteBtn.Enabled := ConEditBtn.Enabled;
  ConAddGrBtn.Enabled := ConEditBtn.Enabled;
  ConRemGrBtn.Enabled := ConEditBtn.Enabled and (GroupsLB.SelCount > 1);
end;

procedure TMainFormCont.UpdateContacts(Group: TYGroup);
var
  I: Integer;
begin
  with ContactsLB.Items, YContacts do
  begin
    if GroupsLB.ItemIndex > 0 then
    begin
      GroupBox1.Caption := 'Contacts (' + GroupsLB.Items[GroupsLB.ItemIndex] + ')';
    end
    else
      GroupBox1.Caption := 'Contacts (All contacts)';
    Clear;
    for I := 0 to ContactsCount - 1 do
      if (Group = nil) or Contacts[I].InGroup(Group) then
      begin
        if Contacts[I].ItemState <> ysDeleted then
          AddObject(Contacts[I].Title, Contacts[I]);
      end;
  end;
  UpdateConButtons;
end;

procedure TMainFormCont.ContactsLBClick(Sender: TObject);
var
  I, Index: Integer;
  Contact: TYContact;
begin
  GroupsLB.MultiSelect := true;
  GrEditBtn.Enabled := false;
  GrDeleteBtn.Enabled := false;
  with ContactsLB do
    Contact := TYContact(Items.Objects[ItemIndex]);
  with GroupsLB do
  begin
    Index := ItemIndex;
    ClearSelection;
    Selected[0] := true;
    for I := 1 to Count - 1 do
      if Contact.InGroup(TYGroup(Items.Objects[I])) then
        Selected[I] := True;
    ItemIndex := Index;
  end;
  UpdateConButtons;
end;

procedure TMainFormCont.ConEditBtnClick(Sender: TObject);
begin
  with ContactsLB, EditContForm do
  begin
    Contacts := YContacts;
    Group := nil;
    Contact := TYContact(Items.Objects[ItemIndex]);
    if ShowModal = mrOK then
    begin
      Items[ItemIndex] := Contact.Title;
      StoreBtn.Enabled := True;
    end;
  end;
end;

procedure TMainFormCont.ConAddBtnClick(Sender: TObject);
begin
  with GroupsLB do
    if SelCount > 1 then
    begin
      ClearSelection;
      GroupsLBClick(Sender);
    end;
  with ContactsLB, EditContForm do
  begin
    Contacts := YContacts;
    if GroupsLB.ItemIndex > 0 then
      Group := TYGroup(GroupsLB.Items.Objects[GroupsLB.ItemIndex])
    else
      Group := nil;
    Contact := nil;
    if ShowModal = mrOK then
    begin
      Items.AddObject(Contact.Title, Contact);
      StoreBtn.Enabled := True;
    end;
  end;
end;

procedure TMainFormCont.ConDeleteBtnClick(Sender: TObject);
begin
  with ContactsLB do
  begin
    TYContact(Items.Objects[ItemIndex]).Delete;
    Items.Delete(ItemIndex);
    StoreBtn.Enabled := True;
  end;
end;

procedure TMainFormCont.UpdateGroups;
var
  I: Integer;
begin
  with GroupsLB.Items, YContacts do
  begin
    Clear;
    Add('All contacts (not a group)');
    for I := 0 to GroupsCount - 1 do
      if Groups[I].ItemState <> ysDeleted then
        AddObject(Groups[I].Name, Groups[I]);
  end;
  GroupsLB.ItemIndex := -1;
  UpdateGrButtons;
end;

procedure TMainFormCont.GroupsLBClick(Sender: TObject);
begin
  UpdateGrButtons;
  with GroupsLB do
  begin
    MultiSelect := false;
    UpdateContacts(TYGroup(Items.Objects[ItemIndex]));
  end;
end;

procedure TMainFormCont.UpdateGrButtons;
begin
  with GroupsLB do
    GrEditBtn.Enabled := (ItemIndex > 0);
  GrDeleteBtn.Enabled := GrEditBtn.Enabled;
end;

procedure TMainFormCont.GrAddBtnClick(Sender: TObject);
begin
  with EditGroupForm, GroupsLB do
  begin
    ClearSelection;
    Contacts := YContacts;
    Group := nil;
    if ShowModal = mrOK then
    begin
      Items.AddObject(Group.Name, Group);
      StoreBtn.Enabled := True;
    end;
  end;
end;

procedure TMainFormCont.GrEditBtnClick(Sender: TObject);
begin
  with EditGroupForm, GroupsLB do
  begin
    Contacts := YContacts;
    Group := TYGroup(Items.Objects[ItemIndex]);
    if ShowModal = mrOK then
    begin
      Items[ItemIndex] := Group.Name;
      StoreBtn.Enabled := True;
    end;
  end;
end;

procedure TMainFormCont.GrDeleteBtnClick(Sender: TObject);
begin
  with GroupsLB do
  begin
    TYGroup(Items.Objects[ItemIndex]).Delete;
    Items.Delete(ItemIndex);
    StoreBtn.Enabled := True;
  end;
end;

procedure TMainFormCont.GroupsLBDblClick(Sender: TObject);
begin
  if GrEditBtn.Enabled then
    GrEditBtnClick(Sender);
end;

procedure TMainFormCont.ContactsLBDblClick(Sender: TObject);
begin
  if ConEditBtn.Enabled then
    ConEditBtnClick(Sender);
end;

procedure TMainFormCont.ConAddGrBtnClick(Sender: TObject);
var
  I: Integer;
  Contact: TYContact;
begin
  with ContactsLB do
    Contact := TYContact(Items.Objects[ItemIndex]);
  with SelGroupForm, SelGroupLB, YContacts do
  begin
    Items.Clear;
    for I := 0 to GroupsCount - 1 do
      if not Contact.InGroup(Groups[I]) then
        Items.AddObject(Groups[I].Name, Groups[I]);
    if ShowModal = mrOK then
    begin
      Contact.AddToGroup(TYGroup(Items.Objects[ItemIndex]));
      ContactsLBClick(Sender);
      StoreBtn.Enabled := True;
    end;
  end;
end;

procedure TMainFormCont.ConRemGrBtnClick(Sender: TObject);
var
  I: Integer;
  Contact: TYContact;
  CurGroup: TYGroup;
begin
  with GroupsLB do
    if ItemIndex <> -1 then
      CurGroup := TYGroup(Items.Objects[ItemIndex])
    else
      CurGroup := nil;
  with ContactsLB do
    Contact := TYContact(Items.Objects[ItemIndex]);
  with SelGroupForm, SelGroupLB, YContacts do
  begin
    Items.Clear;
    for I := 0 to GroupsCount - 1 do
      if Contact.InGroup(Groups[I]) then
        Items.AddObject(Groups[I].Name, Groups[I]);
    if ShowModal = mrOK then
    begin
      Contact.RemoveFromGroup(TYGroup(Items.Objects[ItemIndex]));
      ContactsLBClick(Sender);
      if TYGroup(Items.Objects[ItemIndex]) = CurGroup then
        with ContactsLB do
        begin
          Items.Delete(ItemIndex);
          UpdateConButtons;
        end;
      StoreBtn.Enabled := True;
    end;
  end;
end;

procedure TMainFormCont.CheckBox1Click(Sender: TObject);
begin
  ProxyEd.Enabled := CheckBox1.Checked;
  PortEd.Enabled := ProxyEd.Enabled;
end;

procedure TMainFormCont.StoreBtnClick(Sender: TObject);
begin
  if not YContacts.Store then
    Exit;
  StoreBtn.Enabled := False;
  UpdateGroups;
  UpdateContacts(nil);
end;

end.

